#include <ADuC7026.h>
#include <I2C_Master.h>
#include <stdio.h>


unsigned char i;

unsigned char ReadData[8], WriteData[8];

unsigned int DevData[3];

unsigned char 		DevID;

unsigned char   	DevInterrupt;

unsigned char   	IsStart;


void PutChar( unsigned char Data)
{
	COMTX=Data;	
	while(!(0x020==(COMSTA0 & 0x020)))	{;}		
}


void IRQ_Handler() __irq
{
	unsigned char UartDataReceived, UartInterrupt;
	unsigned char i;

	if((IRQSTA&SPM4_IO_BIT)==SPM4_IO_BIT) 	//Button
	{
		I2C_READ_REGISTER(0x53,0x32,6);

		DevData[0]=ReadData[1]&0x1F;
		DevData[0]=(DevData[0]<<8)|ReadData[0];
		DevData[1]=ReadData[3]&0x1F;
		DevData[1]=(DevData[1]<<8)|ReadData[2];
		DevData[2]=ReadData[5]&0x1F;
		DevData[2]=(DevData[2]<<8)|ReadData[4];

		for(i=0; i<3; i++)
		{
			if(DevData[i] < 4096)
			{
				DevData[i] = DevData[i] + 4096;	
			}
			else //if(DevData[i] >= 4096)
			{
				DevData[i] = DevData[i] - 4096;	
			}
		}

		ReadData[0]=DevData[0]>>8;
		ReadData[1]=DevData[0]&0xFF;
		ReadData[2]=DevData[1]>>8;
		ReadData[3]=DevData[1]&0xFF;
		ReadData[4]=DevData[2]>>8;
		ReadData[5]=DevData[2]&0xFF;

		for(i=0; i<6; i++)
		{
			PutChar(ReadData[i]);	
		}


	}	
	else if((IRQSTA&UART_BIT)==UART_BIT)	//UART
	{
		UartInterrupt =COMIID0 ;
		if(UartInterrupt==0x04)//Has Received a Data
		{
			UartDataReceived=COMRX;
	
			if(UartDataReceived==0xAA)	
			{
				//check DevID command
				COMTX=DevID;	
			}
			else if(UartDataReceived==0x55)	
			{
				IsStart=1;
			}
			else if(UartDataReceived==0xA5)	
			{
				IsStart=0;
			}
		}
	}
}


void UART_Initiate()
{
	//--------------------------------------------------------------------
	//Initiate the UART Port to 115200bps
	POWKEY1 = 0x01;				//Start PLL setting,changeless
	POWCON=0x00;
	POWKEY2 = 0xF4;				//Finish PLL setting,changeless
	// Setup tx & rx pins on P1.0 and P1.1
	GP1CON = 0x2211;				// I2C on P1.2 and P1.3	  Setup tx & rx pins on P1.0 and P1.1 for UART
 
	 // CD = 0 !		 
	COMCON0 = 0x80;					// Setting DLAB
   	COMDIV0 = 0x0B;					// Setting DIV0 and DIV1 to DL calculated
	COMDIV1 = 0x00;

   	COMCON0 = 0x07;					// Clearing DLAB
	// fractional divider
  	COMDIV2 = 0x883E;			  	// M=1
									// N=01101010101  =853
									// M+N/2048	 =1.4165
									//41.78MHz/(16*2*2^CD*DL*(M+N/2048))	 //CD=0  DL=0B=11
									//115.2Kbps  M+N/2048 =1.0303	M=1, N=	 62=0x3EH=000 0011 1110
									//comdiv2=0x883E

	//Enable UART interrupt
	COMIEN0=0x03;					
	IRQEN |= UART_BIT;	
										
}

void I2C1_Initiate() 
{
	//--------------------------------------------------------------------
	//Initiate the I2C1 Port to 100kbps
	GP1CON = 0x2211;				// I2C on P1.2 and P1.3	  Setup tx & rx pins on P1.0 and P1.1 for UART
	I2C1CFG = 0x82;		  			// Master Enable & Enable Generation of Master Clock
	
	// I2C-Master setup
 	I2C1DIV = 0x3232;				// 0x3232 = 400kHz
									// 0xCFCF = 100kHz	

	//Enable I2C1 Master Interupt
	FIQEN |= SM_MASTER1_BIT;	

					
	//I2C0 Port Initiate OK...

}

void ADuC7026_Initiate(void)
{

	UART_Initiate();
	I2C1_Initiate();

	GP4DAT = GP4DAT | 0x01000000; //Grouding the SDO, I2C address for writing and reading is 0xA6 and 0xA7
	//GP4DAT = GP4DAT | 0x01010000; //Highing the SDO, I2C address for writing and reading is 0x3A and 0x3B

	IRQEN  |=SPM4_IO_BIT;
	
}






void  main(void)
{

    ADuC7026_Initiate();          //ADuC7026 initialization


	//ADXL345_INIT	
	WriteData[0]=0xFF;
	WriteData[1]=0x05;
	WriteData[2]=0xFF;
	I2C_WRITE_REGISTER(0x53,0x1E,3);


	WriteData[0]=0x0A;
	WriteData[1]=0x00;
	WriteData[2]=0x80;
	WriteData[3]=0x00;
	I2C_WRITE_REGISTER(0x53,0x2C,4);

	WriteData[0]=0x0B;
	I2C_WRITE_REGISTER(0x53,0x31,1);

	I2C_READ_REGISTER(0x53,0x00,1);	// Get Device ID
	DevID=ReadData[0];

	I2C_READ_REGISTER(0x53,0x30,1);	//clear interrupt;
	DevInterrupt=ReadData[0];
	

	IsStart=0xFF;


	while(1)
	{
		if(IsStart==1)
		{
			IsStart=0xFF;
			WriteData[0]=0x08;
			I2C_WRITE_REGISTER(0x53,0x2D,1);	//Power CTL: measure mode
		}
		else if(IsStart==0)
		{
			IsStart=0xFF;
			WriteData[0]=0x00;
			I2C_WRITE_REGISTER(0x53,0x2D,1);	//Power CTL: measure mode
		}
	}
}